@@ -28,6 +28,7 @@
 
 #include <kglobal.h>
 #include <kdebug.h>
+#include <k3process.h>
 
 #include "disks.moc"
 
@@ -191,6 +192,7 @@ QString DiskEntry::iconName()
    return guessIconName();
 }
 
+//TODO: Need porting to solid
 QString DiskEntry::guessIconName()
 {
   QString iconName;
@@ -210,7 +212,7 @@ QString DiskEntry::guessIconName()
     else if (mountPoint().contains("zip",Qt::CaseInsensitive)) iconName+="zip"; //TODO
     else if (fsType().contains("nfs",Qt::CaseInsensitive)) iconName+="nfs"; //TODO
     else iconName+="drive-harddisk";
-    //mounted() ? iconName+="_mount" : iconName+="_unmount";
+    ///mounted() ? iconName+="_mount" : iconName+="_unmount";
 //    if ( !mountOptions().contains("user",Qt::CaseInsensitive) )
 //      iconName.prepend("root_"); // special root icon, normal user can't mount
 
