@@ -21,28 +21,28 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-
 #ifndef __DISKS_H__
 #define __DISKS_H__
 
-#include <qobject.h>
-#include <qstring.h>
-#include <qlabel.h>
-#include <qpushbutton.h>
-#include <qprogressbar.h>
-#include <qfile.h>
+#include <QtCore/QObject>
+
+#include <QtGui/QLabel>
+#include <QtGui/QPushButton>
+#include <QtGui/QProgressBar>
+#include <QtCore/QFile>
 
 #include <kio/global.h>
-#include <kprogress.h>
-#include <kprocess.h>
 #include <klocale.h>
 
+class K3Process;
+class K3ShellProcess;
+
 class DiskEntry : public QObject
 {
   Q_OBJECT
 public:
-  DiskEntry(QObject *parent=0, const char *name=0);
-  DiskEntry(const QString & deviceName, QObject *parent=0, const char *name=0);
+  explicit DiskEntry(QObject *parent=0, const char *name=0);
+  explicit DiskEntry(const QString & deviceName, QObject *parent=0, const char *name=0);
   ~DiskEntry();
   QString lastSysError() {return sysStringErrOut; }
   QString deviceName() const { return device; }
@@ -79,11 +79,11 @@ public:
   int kBSize() const { return size; }
   QString iconName();
   QString realIconName() { return icoName; }
-  QString prettyKBSize() const { return KIO::convertSizeFromKB(size); }
+  QString prettyKBSize() const { return KIO::convertSizeFromKiB(size); }
   int kBUsed() const { return used; }
-  QString prettyKBUsed() const { return KIO::convertSizeFromKB(used); }
+  QString prettyKBUsed() const { return KIO::convertSizeFromKiB(used); }
   int kBAvail() const  { return avail; }
-  QString prettyKBAvail() const { return KIO::convertSizeFromKB(avail); }
+  QString prettyKBAvail() const { return KIO::convertSizeFromKiB(avail); }
   float percentFull() const;
 
 signals:
@@ -118,14 +118,14 @@ public slots:
   QString guessIconName();
 
 private slots:
-   void receivedSysStdErrOut(KProcess *, char *data, int len);
+   void receivedSysStdErrOut(K3Process *, char *data, int len);
 
 private:
-  void init();
+  void init(const char *name);
   int sysCall(const QString & command);
   QString prettyPrint(int kBValue) const;
 
-  KShellProcess     *sysProc;
+  K3ShellProcess     *sysProc;
   QString           sysStringErrOut;
   bool              readingSysStdErrOut;
 
