@@ -21,7 +21,6 @@
 #include <QDesktopWidget>
 #include <QKeySequence>
 #include <QTextStream>
-#include <QCloseEvent>
 #include <QClipboard>
 #include <QPainter>
 #include <QPrinter>
@@ -74,7 +73,6 @@
 #include "newkey.h"
 #include "kgpg.h"
 #include "kgpgeditor.h"
-#include "kgpgview.h"
 #include "keyexport.h"
 #include "kgpgrevokewidget.h"
 #include "keyservers.h"
@@ -90,6 +88,12 @@
 #include "keylistproxymodel.h"
 #include "keytreeview.h"
 #include "groupedit.h"
+#include "kgpgchangekey.h"
+#include "kgpgdeluid.h"
+#include "kgpgaddphoto.h"
+#include "kgpgadduid.h"
+#include "kgpggeneratekey.h"
+#include "kgpgdelkey.h"
 
 using namespace KgpgCore;
 
@@ -118,7 +122,7 @@ KeysManager::KeysManager(QWidget *parent)
     KStandardAction::findNext(this, SLOT(findNextKey()), actionCollection());
     actionCollection()->addAction(KStandardAction::Preferences, "options_configure", this, SLOT(showOptions()));
 
-    QAction *action = 0;
+    KAction *action = 0;
 
     action = actionCollection()->addAction( "default" );
     connect(action, SIGNAL(triggered(bool)), SLOT(slotDefaultAction()));
@@ -178,19 +182,19 @@ KeysManager::KeysManager(QWidget *parent)
     connect(editKey, SIGNAL(triggered(bool)), SLOT(slotedit()));
     editKey->setShortcut(QKeySequence(Qt::ALT+Qt::Key_Return));
 
-    QAction *generateKey = actionCollection()->addAction("key_gener");
+    KAction *generateKey = actionCollection()->addAction("key_gener");
     generateKey->setIcon(KIcon("key-generate-pair"));
     generateKey->setText(i18n("&Generate Key Pair..."));
     connect(generateKey, SIGNAL(triggered(bool)), SLOT(slotGenerateKey()));
     generateKey->setShortcuts(KStandardShortcut::shortcut(KStandardShortcut::New));
 
-    QAction *exportPublicKey = actionCollection()->addAction("key_export");
+    KAction *exportPublicKey = actionCollection()->addAction("key_export");
     exportPublicKey->setIcon(KIcon("document-export-key"));
     exportPublicKey->setText(i18n("E&xport Public Keys..."));
     connect(exportPublicKey, SIGNAL(triggered(bool)), SLOT(slotexport()));
     exportPublicKey->setShortcuts(KStandardShortcut::shortcut(KStandardShortcut::Copy));
 
-    QAction *importKey = actionCollection()->addAction("key_import");
+    KAction *importKey = actionCollection()->addAction("key_import");
     importKey->setIcon(KIcon("document-import-key"));
     importKey->setText(i18n("&Import Key..."));
     connect(importKey, SIGNAL(triggered(bool)), SLOT(slotPreImportKey()));
@@ -414,10 +418,6 @@ KeysManager::KeysManager(QWidget *parent)
     connect(photoProps, SIGNAL(triggered(int)), this, SLOT(slotSetPhotoSize(int)));
     connect(trustProps, SIGNAL(triggered(int)), this, SLOT(slotSetTrustFilter(int)));
 
-    // get all keys data
-    setupGUI(KXmlGuiWindow::Create | Save | ToolBar | StatusBar | Keys, "keysmanager.rc");
-    toolBar()->addSeparator();
-
     QLabel *searchLabel = new QLabel(i18n("Search: "), this);
     m_listviewsearch = new KLineEdit(this);
     m_listviewsearch->setClearButtonShown(true);
@@ -428,18 +428,19 @@ KeysManager::KeysManager(QWidget *parent)
     searchLayout->addWidget(m_listviewsearch);
     searchLayout->addStretch();
 
-    KAction *serchLineAction = new KAction(i18n("Search Line"), this);
+    KAction *serchLineAction = new KAction(i18nc("Name of the action that is a search line, shown for example in the toolbar configuration dialog", "Search Line"), this);
     actionCollection()->addAction("search_line", serchLineAction);
     serchLineAction->setDefaultWidget(searchWidget);
 
-    toolBar()->addAction(actionCollection()->action("search_line"));
-
     action = actionCollection()->addAction("search_focus");
-    action->setText(i18n("Filter Search"));
+    action->setText(i18nc("Name of the action that gives the focus to the search line", "Focus Search Line"));
     connect(action, SIGNAL(triggered(bool) ), m_listviewsearch, SLOT(setFocus()));
     action->setShortcut(QKeySequence(Qt::Key_F6));
     connect(m_listviewsearch, SIGNAL(textChanged(const QString &)), iproxy, SLOT(setFilterFixedString(const QString &)));
 
+    // get all keys data
+    setupGUI(KXmlGuiWindow::Create | Save | ToolBar | StatusBar | Keys, "keysmanager.rc");
+
     sTrust->setChecked(KGpgSettings::showTrust());
     iview->setColumnHidden(2, !KGpgSettings::showTrust());
     sSize->setChecked(KGpgSettings::showSize());
@@ -462,11 +463,14 @@ KeysManager::KeysManager(QWidget *parent)
     setAutoSaveSettings(cg, true);
     applyMainWindowSettings(cg);
 
-    s_kgpgEditor = new KgpgEditor(parent, Qt::WType_Dialog, qobject_cast<KAction *>(actionCollection()->action("go_default_key"))->shortcut(), true);
+    s_kgpgEditor = new KgpgEditor(parent, imodel, Qt::Dialog, qobject_cast<KAction *>(actionCollection()->action("go_default_key"))->shortcut(), true);
     connect(s_kgpgEditor, SIGNAL(refreshImported(QStringList)), imodel, SLOT(refreshKeys(QStringList)));
     connect(this, SIGNAL(fontChanged(QFont)), s_kgpgEditor, SLOT(slotSetFont(QFont)));
 
     pop = NULL;
+    m_genkey = NULL;
+    m_adduid = NULL;
+    m_delkey = NULL;
 }
 
 KeysManager::~KeysManager()
@@ -477,15 +481,20 @@ KeysManager::~KeysManager()
 
 void KeysManager::slotGenerateKey()
 {
+	if (m_genkey) {
+		KMessageBox::error(this, i18n("Another key generation operation is still in progress.\nPlease wait a moment until this operation is complete."), i18n("Generating new key pair"));
+		return;
+	}
+
     KgpgKeyGenerate *kg = new KgpgKeyGenerate(this);
     if (kg->exec() == QDialog::Accepted)
     {
         if (!kg->isExpertMode())
         {
-            KgpgInterface *interface = new KgpgInterface();
-            connect(interface, SIGNAL(generateKeyStarted(KgpgInterface*)), this, SLOT(slotGenerateKeyProcess(KgpgInterface*)));
-            connect(interface, SIGNAL(generateKeyFinished(int, KgpgInterface*, QString, QString, QString)), this, SLOT(slotGenerateKeyDone(int, KgpgInterface*, QString, QString, QString)));
-            interface->generateKey(kg->name(), kg->email(), kg->comment(), kg->algo(), kg->size(), kg->expiration(), kg->days());
+		m_genkey = new KGpgGenerateKey(this, kg->name(), kg->email(), kg->comment(), kg->algo(), kg->size(), kg->expiration(), kg->days());
+		connect(m_genkey, SIGNAL(generateKeyStarted()), SLOT(slotGenerateKeyProcess()));
+		connect(m_genkey, SIGNAL(done(int)), SLOT(slotGenerateKeyDone(int)));
+		m_genkey->start();
         }
         else
         {
@@ -497,8 +506,12 @@ void KeysManager::slotGenerateKey()
 
             QProcess *genKeyProc = new QProcess(this);
             genKeyProc->start(terminalApp, args);
-            genKeyProc->waitForFinished();
-            refreshkey();
+	    if (!genKeyProc->waitForStarted(-1)) {
+		KMessageBox::error(this, i18n("Generating new key pair"), i18n("Can not start \"konsole\" application for expert mode."));
+	    } else {
+		genKeyProc->waitForFinished(-1);
+		refreshkey();
+	    }
         }
     }
 
@@ -512,7 +525,7 @@ void KeysManager::showKeyManager()
 
 void KeysManager::slotOpenEditor()
 {
-    KgpgEditor *kgpgtxtedit = new KgpgEditor(this, Qt::Window, qobject_cast<KAction *>(actionCollection()->action("go_default_key"))->shortcut());
+    KgpgEditor *kgpgtxtedit = new KgpgEditor(this, imodel, Qt::Window, qobject_cast<KAction *>(actionCollection()->action("go_default_key"))->shortcut());
     kgpgtxtedit->setAttribute(Qt::WA_DeleteOnClose);
 
     connect(kgpgtxtedit, SIGNAL(refreshImported(QStringList)), imodel, SLOT(refreshKeys(QStringList)));
@@ -539,7 +552,7 @@ void KeysManager::changeMessage(const QString &msg, const int nb, const bool kee
     }
 }
 
-void KeysManager::slotGenerateKeyProcess(KgpgInterface *)
+void KeysManager::slotGenerateKeyProcess()
 {
     pop = new KPassivePopup(this);
     pop->setTimeout(0);
@@ -566,9 +579,8 @@ void KeysManager::slotGenerateKeyProcess(KgpgInterface *)
     changeMessage(i18n("Generating New Key..."), 0, true);
 }
 
-void KeysManager::slotGenerateKeyDone(int res, KgpgInterface *interface, const QString &name, const QString &email, const QString &fingerprint)
+void KeysManager::slotGenerateKeyDone(int res)
 {
-    interface->deleteLater();
     changeMessage(i18nc("Application ready for user input", "Ready"), 0);
 
     delete pop;
@@ -602,7 +614,7 @@ void KeysManager::slotGenerateKeyDone(int res, KgpgInterface *interface, const Q
         KMessageBox::error(this, infotext, infomessage);
     }
     else
-    if (res != 2)
+    if (res != 0)
     {
         QString infomessage = i18n("Generating new key pair");
         QString infotext = i18n("gpg process did not finish. Cannot generate a new key pair.");
@@ -619,7 +631,9 @@ void KeysManager::slotGenerateKeyDone(int res, KgpgInterface *interface, const Q
         keyCreated->setModal(true);
 
         newKey *page = new newKey(keyCreated);
-        page->TLname->setText("<b>" + name + "</b>");
+        page->TLname->setText("<b>" + m_genkey->getName() + "</b>");
+
+        QString email(m_genkey->getEmail());
         page->TLemail->setText("<b>" + email + "</b>");
 
 	QString revurl;
@@ -634,6 +648,7 @@ void KeysManager::slotGenerateKeyDone(int res, KgpgInterface *interface, const Q
         else
             page->kURLRequester1->setUrl(revurl + email.section(" ", 0, 0) + ".revoke");
 
+        QString fingerprint(m_genkey->getFingerprint());
         page->TLid->setText("<b>" + fingerprint.right(8) + "</b>");
         page->LEfinger->setText(fingerprint);
         page->CBdefault->setChecked(true);
@@ -661,6 +676,8 @@ void KeysManager::slotGenerateKeyDone(int res, KgpgInterface *interface, const Q
             connect(revKeyProcess, SIGNAL(revokecertificate(QString)), this, SLOT(doPrint(QString)));
         }
     }
+    m_genkey->deleteLater();
+    m_genkey = NULL;
 }
 
 void KeysManager::slotShowTrust()
@@ -776,30 +793,31 @@ void KeysManager::refreshKeyFromServer()
     }
 
     kServer = new KeyServer(this, false);
-    connect(kServer, SIGNAL(importFinished(QStringList)), imodel, SLOT(refreshKeys(const QStringList &)));
+    connect(kServer, SIGNAL(importFinished(QStringList)), imodel, SLOT(refreshKeys(QStringList)));
     kServer->refreshKeys(keyIDS);
 }
 
 void KeysManager::slotDelUid()
 {
-    KGpgNode *nd = iview->selectedNode();
-    Q_ASSERT(nd->getType() == ITYPE_UID);
+	KGpgUidNode *nd = iview->selectedNode()->toUidNode();
 
-    QProcess *process = new QProcess(this);
-    KConfigGroup config(KGlobal::config(), "General");
-    QString terminalApp = config.readPathEntry("TerminalApplication", "konsole");
-    QStringList args;
-    args << "-e" << KGpgSettings::gpgBinaryPath();
-    args << "--edit-key" << nd->getParentKeyNode()->getId() << "uid" << nd->getId() << "deluid";
-    process->start(terminalApp, args);
-    process->waitForFinished();
-    imodel->refreshKey(nd->getParentKeyNode()->getId());
+	m_deluid = new KGpgDelUid(this, nd->getParentKeyNode()->getId(), nd->getId());
+
+	connect(m_deluid, SIGNAL(done(int)), SLOT(slotDelUidDone(int)));
+	m_deluid->start();
+}
+
+void KeysManager::slotDelUidDone(int result)
+{
+	m_deluid->deleteLater();
+	m_deluid = NULL;
+	// FIXME: do something useful with result
+	Q_UNUSED(result)
 }
 
 void KeysManager::slotPrimUid()
 {
-    KGpgNode *nd = iview->selectedNode();
-    Q_ASSERT(nd->getType() == ITYPE_UID);
+    KGpgUidNode *nd = iview->selectedNode()->toUidNode();
 
     QProcess *process = new QProcess(this);
     KConfigGroup config(KGlobal::config(), "General");
@@ -809,7 +827,7 @@ void KeysManager::slotPrimUid()
     args << "--edit-key" << nd->getParentKeyNode()->getId() << "uid" << nd->getId() << "primary" << "save";
     process->start(terminalApp, args);
     process->waitForFinished();
-    imodel->refreshKey(nd->getParentKeyNode()->getId());
+    imodel->refreshKey(nd->getParentKeyNode()->toKeyNode());
 }
 
 void KeysManager::slotregenerate()
@@ -848,13 +866,18 @@ void KeysManager::slotregenerate()
 
 void KeysManager::slotAddUid()
 {
+	if (m_adduid) {
+		KMessageBox::error(this, i18n("Another operation is still in progress.\nPlease wait a moment until this operation is complete."), i18n("Add New User Id"));
+		return;
+	}
+
     addUidWidget = new KDialog(this );
     addUidWidget->setCaption( i18n("Add New User Id") );
     addUidWidget->setButtons(  KDialog::Ok | KDialog::Cancel );
     addUidWidget->setDefaultButton(  KDialog::Ok );
     addUidWidget->setModal( true );
     addUidWidget->enableButtonOk(false);
-    AddUid *keyUid = new AddUid(0);
+    AddUid *keyUid = new AddUid(addUidWidget);
     addUidWidget->setMainWidget(keyUid);
     //keyUid->setMinimumSize(keyUid->sizeHint());
     keyUid->setMinimumWidth(300);
@@ -863,19 +886,18 @@ void KeysManager::slotAddUid()
     if (addUidWidget->exec() != QDialog::Accepted)
         return;
 
-    KgpgInterface *addUidProcess = new KgpgInterface();
-    connect(addUidProcess,SIGNAL(addUidFinished(int, KgpgInterface*)), this ,SLOT(slotAddUidFin(int, KgpgInterface*)));
-    addUidProcess->addUid(iview->selectedNode()->getId(), keyUid->kLineEdit1->text(), keyUid->kLineEdit2->text(), keyUid->kLineEdit3->text());
+    m_adduid = new KGpgAddUid(this, iview->selectedNode()->getId(), keyUid->kLineEdit1->text(), keyUid->kLineEdit2->text(), keyUid->kLineEdit3->text());
+    connect(m_adduid, SIGNAL(done(int)), SLOT(slotAddUidFin(int)));
+    m_adduid->start();
 }
 
-void KeysManager::slotAddUidFin(int res, KgpgInterface *interface)
+void KeysManager::slotAddUidFin(int res)
 {
-    // TODO tester les res
-    kDebug(2100) << "Resultat : " << res ;
-    interface->deleteLater();
-    KGpgNode *nd = iview->selectedNode();
-    Q_ASSERT(!(nd->getType() & ~ITYPE_PAIR));
-    imodel->refreshKey(static_cast<KGpgKeyNode *>(nd));
+	// TODO error reporting
+	if (res == 0)
+		imodel->refreshKey(m_adduid->getKeyid());
+	m_adduid->deleteLater();
+	m_adduid = NULL;
 }
 
 void KeysManager::slotAddUidEnable(const QString & name)
@@ -885,47 +907,61 @@ void KeysManager::slotAddUidEnable(const QString & name)
 
 void KeysManager::slotAddPhoto()
 {
-    KgpgLibrary *lib = new KgpgLibrary();
-    connect (lib, SIGNAL(photoAdded()), this, SLOT(slotUpdatePhoto()));
-    lib->addPhoto(iview->selectedNode()->getId());
+	QString mess = i18n("The image must be a JPEG file. Remember that the image is stored within your public key. "
+	"If you use a very large picture, your key will become very large as well! Keeping the image "
+	"close to 240x288 is a good size to use.");
+
+	if (KMessageBox::warningContinueCancel(0, mess) != KMessageBox::Continue)
+		return;
+
+	QString imagepath = KFileDialog::getOpenFileName(KUrl(), "image/jpeg", 0);
+	if (imagepath.isEmpty())
+		return;
+
+	m_addphoto = new KGpgAddPhoto(this, iview->selectedNode()->getId(), imagepath);
+	connect(m_addphoto, SIGNAL(done(int)), SLOT(slotAddPhotoFinished(int)));
+	m_addphoto->start();
+}
+
+void KeysManager::slotAddPhotoFinished(int res)
+{
+	m_addphoto->deleteLater();
+
+	// TODO : add res == 3 (bad passphrase)
+
+	if (res == 0)
+		slotUpdatePhoto();
 }
 
 void KeysManager::slotDeletePhoto()
 {
     KGpgNode *nd = iview->selectedNode();
-    Q_ASSERT(nd->getType() == ITYPE_UAT);
-    KGpgUatNode *und = static_cast<KGpgUatNode *>(nd);
+    KGpgUatNode *und = nd->toUatNode();
     KGpgKeyNode *parent = und->getParentKeyNode();
 
     QString mess = i18n("<qt>Are you sure you want to delete Photo id <b>%1</b><br/>from key <b>%2 &lt;%3&gt;</b>?</qt>",
                         und->getId(), parent->getName(), parent->getEmail());
 
-    /*
-    if (KMessageBox::warningContinueCancel(this, mess, QString(), KGuiItem(i18n("Delete"), "edit-delete")) != KMessageBox::Continue)
-        return;
-    */
+	m_deluid = new KGpgDelUid(this, parent->getId(), und->getId());
+	connect(m_deluid, SIGNAL(done(int)), SLOT(slotDelPhotoFinished(int)));
 
-    KgpgInterface *interface = new KgpgInterface();
-    connect(interface, SIGNAL(deletePhotoFinished(int, KgpgInterface*)), this, SLOT(slotDelPhotoFinished(int, KgpgInterface*)));
-    interface->deletePhoto(parent->getId(), und->getId());
+	m_deluid->start();
 }
 
-void KeysManager::slotDelPhotoFinished(int res, KgpgInterface *interface)
+void KeysManager::slotDelPhotoFinished(int res)
 {
-    interface->deleteLater();
+	m_deluid->deleteLater();
 
-    // TODO : add res == 3 (bad passphrase)
-
-    if (res == 2)
-        slotUpdatePhoto();
+	// TODO : add res == 3 (bad passphrase)
 
+	if (res == 0)
+		slotUpdatePhoto();
 }
 
 void KeysManager::slotUpdatePhoto()
 {
 	KGpgNode *nd = iview->selectedNode();
-	Q_ASSERT(!(nd->getType() & ~ITYPE_PAIR));
-	imodel->refreshKey(static_cast<KGpgKeyNode *>(nd));
+	imodel->refreshKey(nd->toKeyNode());
 }
 
 void KeysManager::slotSetPhotoSize(int size)
@@ -1097,15 +1133,6 @@ void KeysManager::slotTip()
     KTipDialog::showTip(this, QString("kgpg/tips"), true);
 }
 
-void KeysManager::closeEvent (QCloseEvent *e)
-{
-    // kapp->ref(); // prevent KXmlGuiWindow from closing the app
-    // KXmlGuiWindow::closeEvent(e);
-    // e->accept();
-    hide();
-    e->ignore();
-}
-
 void KeysManager::showKeyServer()
 {
     KeyServer *ks = new KeyServer(this);
@@ -1266,7 +1293,7 @@ KeysManager::slotMenu(const QPoint &pos)
 		if (!nd->hasChildren())
 			continue;
 
-		KGpgExpandableNode *exnd = static_cast<KGpgExpandableNode *>(nd);
+		KGpgExpandableNode *exnd = nd->toExpandableNode();
 		if (!exnd->wasExpanded()) {
 			unksig = true;
 			break;
@@ -1284,7 +1311,7 @@ KeysManager::slotMenu(const QPoint &pos)
 	if (itype == ITYPE_SIGN) {
 		bool allunksig = true;
 		for (int i = 0; (i < cnt) && allunksig; i++) {
-			KGpgSignNode *nd = static_cast<KGpgSignNode *>(ndlist.at(i));
+			KGpgSignNode *nd = ndlist.at(i)->toSignNode();
 			allunksig = nd->isUnknown();
 		}
 
@@ -1293,7 +1320,7 @@ KeysManager::slotMenu(const QPoint &pos)
 		m_popupsig->exec(globpos);
 	} else if (itype == ITYPE_UID) {
 		if (cnt == 1) {
-			KGpgKeyNode *knd = static_cast<KGpgUidNode *>(ndlist.at(0))->getParentKeyNode();
+			KGpgKeyNode *knd = ndlist.at(0)->toUidNode()->getParentKeyNode();
 			setPrimUid->setEnabled(knd->getType() & ITYPE_SECRET);
 		}
 		m_popupuid->exec(globpos);
@@ -1370,14 +1397,14 @@ void KeysManager::revokeWidget()
 void KeysManager::slotImportRevoke(const QString &url)
 {
     KgpgInterface *importKeyProcess = new KgpgInterface();
-    connect(importKeyProcess, SIGNAL(importKeyFinished(QStringList)), imodel, SLOT(refreshKeys(QStringList)));
+    connect(importKeyProcess, SIGNAL(importKeyFinished(KgpgInterface *, QStringList)), SLOT(slotRefreshKeys(KgpgInterface *, QStringList)));
     importKeyProcess->importKey(KUrl(url));
 }
 
 void KeysManager::slotImportRevokeTxt(const QString &revokeText)
 {
     KgpgInterface *importKeyProcess = new KgpgInterface();
-    connect(importKeyProcess, SIGNAL(importKeyFinished(QStringList)), imodel, SLOT(refreshKeys(QStringList)));
+    connect(importKeyProcess, SIGNAL(importKeyFinished(KgpgInterface *, QStringList)), SLOT(slotRefreshKeys(KgpgInterface *, QStringList)));
     importKeyProcess->importKey(revokeText);
 }
 
@@ -1452,15 +1479,15 @@ void KeysManager::slotexport()
     dial->setDefaultButton( KDialog::Ok );
     dial->setModal( true );
 
-    KConfig *m_config = new KConfig("kgpgrc", KConfig::SimpleConfig);
-    KConfigGroup gr = m_config->group("Servers");
-    QString servers = gr.readEntry("Server_List");
-    delete m_config;
-
-    QStringList *serverList = new QStringList(servers.split(','));
-
+	QStringList serverList = KGpgSettings::keyServers();
+	serverList.replaceInStrings(QRegExp(" .*"), ""); // Remove kde 3.5 (Default) tag.
+	if (!serverList.isEmpty()) {
+		QString defaultServer = serverList.takeFirst();
+		qSort(serverList);
+		serverList.prepend(defaultServer);
+	}
+    
     KeyExport *page = new KeyExport(dial, serverList);
-    delete serverList;
 
     dial->setMainWidget(page);
     page->newFilename->setUrl(sname);
@@ -1566,8 +1593,7 @@ void KeysManager::slotShowPhoto()
        return;
     }
     KGpgNode *nd = iview->selectedNode();
-    Q_ASSERT(nd->getType() == ITYPE_UAT);
-    KGpgUatNode *und = static_cast<KGpgUatNode *>(nd);
+    KGpgUatNode *und = nd->toUatNode();
     KGpgKeyNode *parent = und->getParentKeyNode();
     KService::Ptr ptr = list.first();
 
@@ -1604,7 +1630,7 @@ void KeysManager::defaultAction(KGpgNode *nd)
 	case ITYPE_GPUBLIC:
 	case ITYPE_GSECRET:
 	case ITYPE_GPAIR:
-		iview->selectNode(static_cast<KGpgRefNode *>(nd)->getRefNode());
+		iview->selectNode(nd->toRefNode()->getRefNode());
 		break;
 	case ITYPE_SECRET:
 		slotregenerate();
@@ -1625,9 +1651,10 @@ KeysManager::showProperties(KGpgNode *n)
 	case ITYPE_PUBLIC:
 	case ITYPE_PAIR:
 		{
-			KGpgKeyNode *k = static_cast<KGpgKeyNode *>(n);
-			KgpgKeyInfo *opts = new KgpgKeyInfo(k->copyKey(), this);
-			connect(opts, SIGNAL(keyNeedsRefresh(const QString &)), imodel, SLOT(refreshKey(const QString &)));
+			KGpgKeyNode *k = n->toKeyNode();
+			KgpgKeyInfo *opts = new KgpgKeyInfo(k, this);
+			connect(opts, SIGNAL(keyNeedsRefresh(KGpgKeyNode *)), imodel, SLOT(refreshKey(KGpgKeyNode *)));
+			connect(opts->keychange, SIGNAL(keyNeedsRefresh(KGpgKeyNode *)), imodel, SLOT(refreshKey(KGpgKeyNode *)));
 			opts->exec();
 			delete opts;
 		}
@@ -1652,18 +1679,17 @@ void KeysManager::keyproperties()
 		break;
 	case ITYPE_PAIR:
 	case ITYPE_PUBLIC: {
-		KGpgKeyNode *kn = static_cast<KGpgKeyNode *>(cur);
-		KgpgKeyInfo *opts = new KgpgKeyInfo(kn->copyKey(), this);
+		KGpgKeyNode *kn = cur->toKeyNode();
+		KgpgKeyInfo *opts = new KgpgKeyInfo(kn, this);
 		connect(opts, SIGNAL(keyNeedsRefresh(const QString &)), imodel, SLOT(refreshKey(const QString &)));
 		opts->exec();
 		delete opts;
 		break;
 	}
-
 	case ITYPE_GPAIR:
 	case ITYPE_GPUBLIC: {
-		KGpgKeyNode *kn = static_cast<KGpgGroupMemberNode *>(cur)->getRefNode();
-		KgpgKeyInfo *opts = new KgpgKeyInfo(kn->copyKey(), this);
+		KGpgKeyNode *kn = cur->toGroupMemberNode()->getRefNode();
+		KgpgKeyInfo *opts = new KgpgKeyInfo(kn, this);
 		connect(opts, SIGNAL(keyNeedsRefresh(const QString &)), imodel, SLOT(refreshKey(const QString &)));
 		opts->exec();
 		delete opts;
@@ -1713,7 +1739,7 @@ void KeysManager::createNewGroup()
                     if ((nd->getTrust() == TRUST_FULL) ||
                         (nd->getTrust() == TRUST_ULTIMATE)) {
                         keysGroup += nd->getId();
-                        keysList.append(static_cast<KGpgKeyNode *>(nd));
+                        keysList.append(nd->toKeyNode());
                     } else
                         badkeys += i18nc("<Name> (<Email>) ID: <KeyId>", "%1 (%2) ID: %3", nd->getName(), nd->getEmail(), nd->getId());
             }
@@ -1764,7 +1790,7 @@ void KeysManager::editGroup()
 			memberids << members.at(i)->getId();
 
 		KgpgInterface::setGpgGroupSetting(nd->getName(), memberids, KGpgSettings::gpgConfigPath());
-		imodel->changeGroup(static_cast<KGpgGroupNode *>(nd), members);
+		imodel->changeGroup(nd->toGroupNode(), members);
 	}
     delete dialogGroupEdit;
 }
@@ -1784,7 +1810,7 @@ void KeysManager::signkey()
 
     if (signList.count() == 1)
     {
-        KGpgKeyNode *nd = static_cast<KGpgKeyNode *>(signList.at(0));
+        KGpgKeyNode *nd = signList.at(0)->toKeyNode();
         QString opt;
 
         if (nd->getEmail().isEmpty())
@@ -1805,7 +1831,7 @@ void KeysManager::signkey()
     {
         QStringList signKeyList;
 		for (int i = 0; i < signList.count(); ++i) {
-			KGpgKeyNode *nd = static_cast<KGpgKeyNode *>(signList.at(i));
+			KGpgKeyNode *nd = signList.at(i)->toKeyNode();
 
 			if (nd->getEmail().isEmpty())
 				signKeyList += i18nc("Name: ID", "%1: %2", nd->getName(), nd->getBeautifiedFingerprint());
@@ -1817,7 +1843,7 @@ void KeysManager::signkey()
             return;
     }
 
-    KgpgSelectSecretKey *opts = new KgpgSelectSecretKey(this, true, signList.count(), imodel);
+    KgpgSelectSecretKey *opts = new KgpgSelectSecretKey(this, imodel, signList.count());
     if (opts->exec() != QDialog::Accepted)
     {
         delete opts;
@@ -1851,9 +1877,9 @@ void KeysManager::signuid()
 		KGpgNode *nd = signList.at(0);
 		KGpgKeyNode *pnd;
 		if (tp & ITYPE_PUBLIC)
-			pnd = static_cast<KGpgKeyNode *>(nd);
+			pnd = nd->toKeyNode();
 		else
-			pnd = static_cast<KGpgKeyNode *>(nd->getParentKeyNode());
+			pnd = nd->getParentKeyNode()->toKeyNode();
 		QString opt;
 
 		if (nd->getEmail().isEmpty())
@@ -1875,9 +1901,7 @@ void KeysManager::signuid()
 		for (int i = 0; i < signList.count(); ++i) {
 			KGpgNode *nd = signList.at(i);
 			KGpgKeyNode *pnd = (nd->getType() & (ITYPE_UID | ITYPE_UAT)) ?
-					static_cast<KGpgKeyNode *>(nd->getParentKeyNode()) :
-					static_cast<KGpgKeyNode *>(nd);
-
+					nd->getParentKeyNode()->toKeyNode() : nd->toKeyNode();
 
 			if (nd->getEmail().isEmpty())
 				signKeyList += i18nc("Name: ID", "%1: %2",
@@ -1892,7 +1916,7 @@ void KeysManager::signuid()
 			return;
 	}
 
-	KgpgSelectSecretKey *opts = new KgpgSelectSecretKey(this, true, signList.count(), imodel);
+	KgpgSelectSecretKey *opts = new KgpgSelectSecretKey(this, imodel, signList.count());
 	if (opts->exec() != QDialog::Accepted) {
 		delete opts;
 		return;
@@ -1919,7 +1943,7 @@ void KeysManager::signLoop()
 		switch (nd->getType()) {
 		case ITYPE_UID:
 		case ITYPE_UAT:
-			keyid = (static_cast<KGpgExpandableNode *>(nd))->getParentKeyNode()->getId();
+			keyid = nd->toExpandableNode()->getParentKeyNode()->getId();
 			uid = nd->getId();
 			break;
 		default:
@@ -1935,8 +1959,7 @@ void KeysManager::signLoop()
 		connect(interface, SIGNAL(signKeyFinished(int, QString, KgpgInterface*)), this, SLOT(signatureResult(int, QString, KgpgInterface*)));
 	} else {
 		signList.clear();
-		for (int i = 0; i < refreshList.count(); i++)
-			imodel->refreshKey(refreshList.at(i));
+		imodel->refreshKeys(refreshList);
 		refreshList.clear();
 	}
 }
@@ -1949,8 +1972,7 @@ void KeysManager::signatureResult(int success, const QString &keyId, KgpgInterfa
 
 	if (success == 2) {
 		KGpgKeyNode *knd = (nd->getType() & (ITYPE_UAT | ITYPE_UID)) ?
-				static_cast<KGpgKeyNode *>(nd->getParentKeyNode()) :
-				static_cast<KGpgKeyNode *>(nd);
+				nd->getParentKeyNode()->toKeyNode() : nd->toKeyNode();
 		if (refreshList.indexOf(knd) == -1)
 			refreshList.append(knd);
 	} else if (success == 1)
@@ -1968,12 +1990,10 @@ void KeysManager::getMissingSigs(QStringList *missingKeys, KGpgExpandableNode *n
 	for (int i = nd->getChildCount() - 1; i >= 0; i--) {
 		KGpgNode *ch = nd->getChild(i);
 		if (ch->hasChildren()) {
-			getMissingSigs(missingKeys, static_cast<KGpgExpandableNode *>(ch));
+			getMissingSigs(missingKeys, ch->toExpandableNode());
 			continue;
-		}
-
-		if (ch->getType() == ITYPE_SIGN) {
-			if (static_cast<KGpgSignNode *>(ch)->isUnknown())
+		} else if (ch->getType() == ITYPE_SIGN) {
+			if (ch->toSignNode()->isUnknown())
 				*missingKeys << ch->getId();
 		}
 	}
@@ -1992,10 +2012,10 @@ void KeysManager::importallsignkey()
 		KGpgNode *nd = sel.at(i);
 
 		if (nd->hasChildren()) {
-			KGpgExpandableNode *en = static_cast<KGpgExpandableNode *>(nd);
+			KGpgExpandableNode *en = nd->toExpandableNode();
 			getMissingSigs(&missingKeys, en);
 		} else if (nd->getType() == ITYPE_SIGN) {
-			KGpgSignNode *sn = static_cast<KGpgSignNode *>(nd);
+			KGpgSignNode *sn = nd->toSignNode();
 
 			if (sn->isUnknown())
 				missingKeys << sn->getId();
@@ -2038,20 +2058,21 @@ void KeysManager::preimportsignkey()
 
 bool KeysManager::importRemoteKey(const QString &keyIDs)
 {
-	KGpgTransaction *xact = new KGpgTransaction();
+	KgpgInterface *iface = new KgpgInterface();
 
 	QStringList kservers = KeyServer::getServerList();
 	if (kservers.isEmpty())
 		return false;
-	connect(xact, SIGNAL(receiveComplete(KGpgTransaction *)), this, SLOT(importRemoteFinished(KGpgTransaction *)));
-	xact->iface->downloadKeys(keyIDs.split(' '), kservers.first(), false, qgetenv("http_proxy"));
+	connect(iface, SIGNAL(downloadKeysFinished(QList<int>, QStringList, bool, QString, KgpgInterface*)), SLOT(importRemoteFinished(QList<int>, QStringList, bool, QString, KgpgInterface*)));
+
+	iface->downloadKeys(keyIDs.split(' '), kservers.first(), false, qgetenv("http_proxy"));
 
 	return true;
 }
 
-void KeysManager::importRemoteFinished(KGpgTransaction *t)
+void KeysManager::importRemoteFinished(QList<int>, QStringList, bool, QString, KgpgInterface *iface)
 {
-	t->deleteLater();
+	iface->deleteLater();
 	imodel->refreshKeys();
 }
 
@@ -2132,9 +2153,9 @@ void KeysManager::delsignatureResult(bool success)
 	if (success) {
 		KGpgNode *nd = iview->selectedNode()->getParentKeyNode();
 
-		while (nd->getType() & ~ITYPE_PAIR)
+		while (!(nd->getType() & ITYPE_PAIR))
 			nd = nd->getParentKeyNode();
-		imodel->refreshKey(nd->getId());
+		imodel->refreshKey(nd->toKeyNode());
 	} else
 		KMessageBox::sorry(this, i18n("Requested operation was unsuccessful, please edit the key manually."));
 }
@@ -2155,7 +2176,7 @@ void KeysManager::slotedit()
     KConfigGroup config(KGlobal::config(), "General");
     *kp << config.readPathEntry("TerminalApplication","konsole");
     *kp << "-e" << KGpgSettings::gpgBinaryPath() <<"--no-secmem-warning" <<"--edit-key" << nd->getId() << "help";
-    terminalkey = static_cast<KGpgKeyNode *>(nd);
+    terminalkey = nd->toKeyNode();
     editKey->setEnabled(false);
 
     connect(kp, SIGNAL(finished(int)), SLOT(slotEditDone(int)));
@@ -2197,9 +2218,59 @@ void KeysManager::doPrint(const QString &txt)
     }
 }
 
+void KeysManager::removeFromGroups(KGpgKeyNode *node)
+{
+	QList<KGpgGroupNode *> groups = node->getGroups();
+	if (groups.count() == 0)
+		return;
+
+	QStringList groupNames;
+
+	for (int i = 0; i < groups.count(); i++)
+		groupNames << groups.at(i)->getName();
+
+	QString ask = i18np("<qt>The key you are deleting is a member of the following key group. Do you want to remove it from this group?</qt>",
+			"<qt>The key you are deleting is a member of the following key groups. Do you want to remove it from these groups?</qt>",
+			groupNames.count());
+	
+	if (KMessageBox::questionYesNoList(this, ask, groupNames, i18n("Delete key")) != KMessageBox::Yes)
+		return;
+
+	QList<KGpgGroupMemberNode *> grefs = node->getGroupRefs();
+
+	bool groupDeleted = false;
+
+	for (int i = 0; i < grefs.count(); i++) {
+		QStringList memberids;
+		KGpgGroupMemberNode *gref = grefs.at(i);
+		KGpgGroupNode *group = gref->getParentKeyNode();
+
+		for (int j = group->getChildCount() - 1; j >= 0; j--) {
+			KGpgGroupMemberNode *rn = group->getChild(j)->toGroupMemberNode();
+			if (rn != gref)
+				memberids << rn->getId();
+		}
+
+		if (!memberids.isEmpty()) {
+			KgpgInterface::setGpgGroupSetting(group->getName(), memberids, KGpgSettings::gpgConfigPath());
+			imodel->deleteFromGroup(group, gref);
+		} else {
+			KgpgInterface::delGpgGroup(group->getName(), KGpgSettings::gpgConfigPath());
+			imodel->delNode(group);
+			groupDeleted = true;
+		}
+	}
+
+	if (groupDeleted) {
+		QStringList groupnames = KgpgInterface::getGpgGroupNames(KGpgSettings::gpgConfigPath());
+		KGpgSettings::setGroups(groupnames.join(","));
+		changeMessage(imodel->statusCountMessage(), 1);
+	}
+}
+
 void KeysManager::deleteseckey()
 {
-	KGpgNode *nd = iview->selectedNode();
+	KGpgKeyNode *nd = iview->selectedNode()->toKeyNode();
 	Q_ASSERT(nd != NULL);
 
     // delete a key
@@ -2213,17 +2284,16 @@ void KeysManager::deleteseckey()
 	if (terminalkey == nd)
 		return;
 	if (delkey != NULL) {
-		KMessageBox::error(this, i18n("Another key is currently deleted. Please wait until the operation is finished."), i18n("Delete key"));
+		KMessageBox::error(this, i18n("Another key delete operation is still in progress.\nPlease wait a moment until this operation is complete."), i18n("Delete key"));
 		return;
 	}
 
-	delkey = static_cast<KGpgKeyNode *>(nd);
-    KProcess *conprocess = new KProcess();
-    KConfigGroup config(KGlobal::config(), "General");
-    *conprocess << config.readPathEntry("TerminalApplication","konsole");
-    *conprocess << "-e" << KGpgSettings::gpgBinaryPath() <<"--no-secmem-warning" << "--delete-secret-and-public-key" << nd->getId();
-    connect(conprocess, SIGNAL(finished(int)), this, SLOT(secretKeyDeleted(int)));
-    conprocess->start();
+	removeFromGroups(nd);
+
+	delkey = nd;
+	m_delkey = new KGpgDelKey(this, nd->getId());
+	connect(m_delkey, SIGNAL(done(int)), SLOT(secretKeyDeleted(int)));
+	m_delkey->start();
 }
 
 void KeysManager::secretKeyDeleted(int retcode)
@@ -2235,10 +2305,17 @@ void KeysManager::secretKeyDeleted(int retcode)
 		KMessageBox::error(this, i18n("Deleting key <b>%1</b> failed.", delkey->getBeautifiedFingerprint()), i18n("Delete key"));
 	}
 	delkey = NULL;
+	m_delkey->deleteLater();
+	m_delkey = NULL;
 }
 
 void KeysManager::confirmdeletekey()
 {
+	if (m_delkey) {
+		KMessageBox::error(this, i18n("Another key delete operation is still in progress.\nPlease wait a moment until this operation is complete."), i18n("Delete key"));
+		return;
+	}
+
 	KgpgCore::KgpgItemType pt;
 	bool same;
 	QList<KGpgNode *> ndlist = iview->selectedNodes(&same, &pt);
@@ -2262,18 +2339,19 @@ void KeysManager::confirmdeletekey()
 	}
 
 	QStringList keysToDelete;
+	QStringList deleteIds;
 	QString secList;
-	KGpgKeyNodeList delkeys;
 
 	bool secretKeyInside = (pt & ITYPE_SECRET);
 	for (int i = 0; i < ndlist.count(); ++i) {
-		KGpgKeyNode *ki = static_cast<KGpgKeyNode *>(ndlist.at(i));
+		KGpgKeyNode *ki = ndlist.at(i)->toKeyNode();
 
 		if (ki->getType() & ITYPE_SECRET) {
 			secList += ki->getNameComment();
 		} else if (ki != terminalkey) {
 			keysToDelete += ki->getNameComment();
-			delkeys << ki;
+			deleteIds << ki->getId();
+			m_delkeys << ki;
 		}
 	}
 
@@ -2290,21 +2368,27 @@ void KeysManager::confirmdeletekey()
         if (result != KMessageBox::Continue)
             return;
 
-    KProcess gp;
-    gp << KGpgSettings::gpgBinaryPath()
-    << "--no-tty"
-    << "--no-secmem-warning"
-    << "--batch"
-    << "--yes"
-    << "--delete-key"
-    << keysToDelete;
+	for (int i = 0; i < ndlist.count(); i++)
+		removeFromGroups(ndlist.at(i)->toKeyNode());
 
-    gp.execute();
+	m_delkey = new KGpgDelKey(this, deleteIds);
+	connect(m_delkey, SIGNAL(done(int)), SLOT(slotDelKeyDone(int)));
+	m_delkey->start();
+}
 
-	for (int i = delkeys.count() - 1; i >= 0; i--)
-		imodel->delNode(delkeys.at(i));
+void KeysManager::slotDelKeyDone(int res)
+{
+	m_delkey->deleteLater();
+	m_delkey = NULL;
 
-    changeMessage(imodel->statusCountMessage(), 1);
+	if (res != 0)
+		return;
+
+	for (int i = m_delkeys.count() - 1; i >= 0; i--)
+		imodel->delNode(m_delkeys.at(i));
+	m_delkeys.clear();
+
+	changeMessage(imodel->statusCountMessage(), 1);
 }
 
 void KeysManager::slotPreImportKey()
@@ -2332,7 +2416,7 @@ void KeysManager::slotPreImportKey()
                 changeMessage(i18n("Importing..."), 0, true);
                 // import from file
                 KgpgInterface *importKeyProcess = new KgpgInterface();
-                connect(importKeyProcess, SIGNAL(importKeyFinished(QStringList)), imodel, SLOT(refreshKeys(QStringList)));
+                connect(importKeyProcess, SIGNAL(importKeyFinished(KgpgInterface *, QStringList)), SLOT(slotRefreshKeys(KgpgInterface *, QStringList)));
                 importKeyProcess->importKey(KUrl(impname));
             }
         }
@@ -2343,7 +2427,7 @@ void KeysManager::slotPreImportKey()
             {
                 changeMessage(i18n("Importing..."), 0, true);
                 KgpgInterface *importKeyProcess = new KgpgInterface();
-                connect(importKeyProcess, SIGNAL(importKeyFinished(QStringList)), imodel, SLOT(refreshKeys(QStringList)));
+                connect(importKeyProcess, SIGNAL(importKeyFinished(KgpgInterface *, QStringList)), SLOT(slotRefreshKeys(KgpgInterface *, QStringList)));
                 importKeyProcess->importKey(keystr);
             }
         }
@@ -2357,15 +2441,17 @@ void KeysManager::refreshkey()
 	changeMessage(imodel->statusCountMessage(), 1);
 }
 
-KGpgTransaction::KGpgTransaction()
-	: iface(new KgpgInterface())
+KGpgItemModel *KeysManager::getModel()
 {
-	connect(iface, SIGNAL(downloadKeysFinished(QList<int>, QStringList, bool, QString, KgpgInterface*)), this, SLOT(slotDownloadKeysFinished(QList<int>, QStringList, bool, QString, KgpgInterface*)));
+	return imodel;
 }
 
-void KGpgTransaction::slotDownloadKeysFinished(QList<int>, QStringList, bool, QString, KgpgInterface*)
+void
+KeysManager::slotRefreshKeys(KgpgInterface *iface, const QStringList &keys)
 {
-	emit receiveComplete(this);
+	iface->deleteLater();
+	if (!keys.isEmpty())
+		imodel->refreshKeys(keys);
 }
 
 #include "keysmanager.moc"
