@@ -106,9 +106,11 @@ signals:
 private slots:
     void slotChangeHome();
     void slotAddKeyServer();
+	void slotChangeEncryptTo();
     void slotDelKeyServer();
     void slotEditKeyServer();
     void slotEditKeyServer(QListWidgetItem *);
+    void slotEnableDeleteServer();
     void slotDefaultKeyServer();
     void updateWidgets();
     void updateWidgetsDefault();
@@ -124,7 +126,6 @@ private:
     bool isValidKeyserver(const QString &);
 
 private:
-    QStringList serverList;
     QStringList names;
     QStringList ids;
     QString alwaysKeyID;
@@ -132,9 +133,10 @@ private:
     QString fileEncryptionKey;
     QString gpgConfigPath;
     QString gpgBinPath;
-    QString keyServer;
-    QString defaultServerList;
-    QString defaultKeyServer;
+    QString keyServer;              // Server stored in gnupg config
+    QStringList serverList;         // Servers stored in kgpgrc
+    QString defaultKeyServer;       // Default keyserver
+    QStringList defaultServerList;  // Default list of servers including the default key server;
     QString defaultConfigPath;
     QString defaultHomePath;
     QString defaultBinPath;
