@@ -132,11 +132,12 @@ KGpgTextInterfacePrivate::signFile(const KUrl &file)
 {
 	*m_process << "--command-fd=0" << "-u" << m_signID.simplified().toLocal8Bit();
 
-	for (QStringList::ConstIterator it = m_gpgopts.begin(); it != m_gpgopts.end(); ++it)
+	for (QStringList::ConstIterator it = m_gpgopts.constBegin(); it != m_gpgopts.constEnd(); ++it)
 		*m_process << QFile::encodeName(*it);
 
-	*m_process << "--output" << QFile::encodeName(file.path() + ".sig");
-	*m_process << "--detach-sig" << QFile::encodeName(file.path());
+	if (m_gpgopts.contains("--detach-sign") && !m_gpgopts.contains("--output"))
+		*m_process << "--output" << QFile::encodeName(file.path() + ".sig");
+	*m_process << QFile::encodeName(file.path());
 
 	m_process->start();
 }
@@ -164,14 +165,14 @@ KGpgTextInterface::encryptText(const QString &text, const QStringList &userids,
 	else
 		d->m_message = text.toUtf8();
 
-	for (QStringList::ConstIterator it = options.begin(); it != options.end(); ++it)
+	for (QStringList::ConstIterator it = options.constBegin(); it != options.constEnd(); ++it)
 		*d->m_process << QFile::encodeName(*it);
 
 	if (userids.isEmpty()) {
 		*d->m_process << "-c";
 	} else {
 		*d->m_process << "-e";
-		for (QStringList::ConstIterator it = userids.begin(); it != userids.end(); ++it)
+		for (QStringList::ConstIterator it = userids.constBegin(); it != userids.constEnd(); ++it)
 			*d->m_process << "--recipient" << *it;
 	}
 
@@ -214,7 +215,7 @@ KGpgTextInterface::encryptTextFin()
 void
 KGpgTextInterface::decryptText(const QString &text, const QStringList &options)
 {
-	for (QStringList::ConstIterator it = options.begin(); it != options.end(); ++it)
+	for (QStringList::ConstIterator it = options.constBegin(); it != options.constEnd(); ++it)
 		*d->m_process << QFile::encodeName(*it);
 
 	*d->m_process << "-d";
@@ -315,7 +316,7 @@ KGpgTextInterface::signText(const QString &text, const QString &userid, const QS
 	else
 		d->m_message = text.toUtf8();
 
-	for (QStringList::ConstIterator it = options.begin(); it != options.end(); ++it)
+	for (QStringList::ConstIterator it = options.constBegin(); it != options.constEnd(); ++it)
 		*d->m_process << QFile::encodeName(*it);
 
 	*d->m_process << "--clearsign" << "-u" << userid;
@@ -402,14 +403,15 @@ KGpgTextInterface::encryptFile(const QStringList &encryptkeys, const KUrl &srcur
 {
 	d->m_file = srcurl;
 
-	for (QStringList::ConstIterator it = options.begin(); it != options.end(); ++it)
+	for (QStringList::ConstIterator it = options.constBegin(); it != options.constEnd(); ++it)
 		*d->m_process << QFile::encodeName(*it);
 
-	*d->m_process << "--output" << QFile::encodeName(desturl.path());
+	if (!options.contains("--output"))
+		*d->m_process << "--output" << QFile::encodeName(desturl.path());
 
 	if (!symetrical) {
 		*d->m_process << "-e";
-		for (QStringList::ConstIterator it = encryptkeys.begin(); it != encryptkeys.end(); ++it)
+		for (QStringList::ConstIterator it = encryptkeys.constBegin(); it != encryptkeys.constEnd(); ++it)
 		*d->m_process << "--recipient" << *it;
 	} else
 		*d->m_process << "-c";
@@ -467,7 +469,7 @@ KGpgTextInterface::KgpgDecryptFileToText(const KUrl &srcUrl, const QStringList &
 {
 	*d->m_process << "--no-batch" << "-o" << "-";
 
-	for (QStringList::ConstIterator it = options.begin(); it != options.end(); ++it)
+	for (QStringList::ConstIterator it = options.constBegin(); it != options.constEnd(); ++it)
 		*d->m_process << QFile::encodeName(*it);
 
 	*d->m_process << "-d" << QFile::encodeName(srcUrl.path());
