@@ -42,6 +42,7 @@
 #include <Q3PtrStack>
 #include <QRegExp>
 
+#include <QRegExpValidator>
 #include <QTimer>
 #include <ktoolinvocation.h>
 #include <kicon.h>
@@ -189,7 +190,7 @@ void KWalletManager::aWalletWasOpened() {
 
 
 void KWalletManager::updateWalletDisplay() {
-    QStringList wl = KWallet::Wallet::walletList();
+    const QStringList wl = KWallet::Wallet::walletList();
     Q3PtrStack<Q3IconViewItem> trash;
 
 	for (Q3IconViewItem *item = _iconView->firstItem(); item; item = item->nextItem()) {
@@ -201,7 +202,7 @@ void KWalletManager::updateWalletDisplay() {
 	trash.setAutoDelete(true);
 	trash.clear();
 
-	for (QStringList::Iterator i = wl.begin(); i != wl.end(); ++i) {
+	for (QStringList::const_iterator i = wl.begin(); i != wl.end(); ++i) {
 		Q3IconViewItem *itm = _iconView->findItem(*i, Q3IconView::ExactMatch | Q3IconView::CaseSensitive);
 		if (!itm) {
 			itm = new KWalletItem(_iconView, *i);
@@ -358,13 +359,11 @@ void KWalletManager::possiblyRescan(const QString& app, const QString& oldOwner,
 	}
 }
 
-
 void KWalletManager::createWallet() {
 	QString n;
 	bool ok;
-	// FIXME: support international names
-	QRegExp regexp("^[A-Za-z0-9]+[A-Za-z0-9_\\s\\-]*$");
 	QString txt = i18n("Please choose a name for the new wallet:");
+	QRegExpValidator validator(QRegExp("^[\\w\\^\\&\\'\\@\\{\\}\\[\\]\\,\\$\\=\\!\\-\\#\\(\\)\\%\\.\\+\\_]+$"), this);
 
 	if (!KWallet::Wallet::isEnabled()) {
 		// FIXME: KMessageBox::warningYesNo(this, i1_8n("KWallet is not enabled.  Do you want to enable it?"), QString(), i18n("Enable"), i18n("Keep Disabled"));
@@ -372,11 +371,8 @@ void KWalletManager::createWallet() {
 	}
 
 	do {
-		n = KInputDialog::getText(i18n("New Wallet"),
-				txt,
-				QString(),
-				&ok,
-				this);
+		n = KInputDialog::getText(i18n("New Wallet"), txt, QString(), &ok, this,
+		                          &validator);
 
 		if (!ok) {
 			return;
@@ -384,17 +380,15 @@ void KWalletManager::createWallet() {
 
 		if (_iconView->findItem(n)) {
 			int rc = KMessageBox::questionYesNo(this, i18n("Sorry, that wallet already exists. Try a new name?"), QString(), KGuiItem(i18n("Try New")), KGuiItem(i18n("Do Not Try")));
-			if (rc == KMessageBox::Yes) {
-				continue;
+			if (rc == KMessageBox::No) {
+				return;
 			}
-			n = QString();
-		} else if (regexp.exactMatch(n)) {
+			n.clear();
+		} else  {
 			break;
-		} else {
-			txt = i18n("Please choose a name that contains only alphanumeric characters:");
 		}
 	} while (true);
-
+	
 	// Small race here - the wallet could be created on us already.
 	if (!n.isEmpty()) {
 		openWallet(n, true);
