@@ -23,15 +23,15 @@
 #include "abstractbytearraycolumnrenderer.h"
 
 // lib
-#include "kdatacursor.h"
+#include "bytearraytableranges.h"
+#include "bytearraytablecursor.h"
 #include "bytearraytablelayout.h"
-#include "kdataranges.h"
 #include "helper.h"
 // ColumnsView
 #include <columnsview.h>
 // Okteta core
 #include <kbookmarkable.h>
-#include <kcharcodec.h>
+#include <charcodec.h>
 // KDE
 #include <KColorScheme>
 // Qt
@@ -51,7 +51,7 @@ static const KPixelX DefaultGroupSpacingWidth = 9;
 static const int DefaultNoOfGroupedBytes = 4;
 
 AbstractByteArrayColumnRenderer::AbstractByteArrayColumnRenderer( ColumnsView *columnsView,
-    KHECore::KAbstractByteArrayModel *byteArrayModel, ByteArrayTableLayout *layout, KDataRanges *ranges )
+    KHECore::AbstractByteArrayModel* byteArrayModel, ByteArrayTableLayout* layout, ByteArrayTableRanges* ranges )
  : ColumnRenderer( columnsView ),
    mByteArrayModel( byteArrayModel ),
    mLayout( layout ),
@@ -71,7 +71,7 @@ AbstractByteArrayColumnRenderer::AbstractByteArrayColumnRenderer( ColumnsView *c
 }
 
 
-void AbstractByteArrayColumnRenderer::set( KHECore::KAbstractByteArrayModel *byteArrayModel )
+void AbstractByteArrayColumnRenderer::set( KHECore::AbstractByteArrayModel *byteArrayModel )
 {
     mByteArrayModel = byteArrayModel;
     mBookmarks = qobject_cast<KHECore::Bookmarkable*>(byteArrayModel );
@@ -252,16 +252,16 @@ int AbstractByteArrayColumnRenderer::magneticLinePositionOfX( KPixelX PX ) const
 }
 
 
-KHE::KSection AbstractByteArrayColumnRenderer::linePositionsOfX( KPixelX PX, KPixelX PW ) const
+KHE::Section AbstractByteArrayColumnRenderer::linePositionsOfX( KPixelX PX, KPixelX PW ) const
 {
     if( !mLinePosLeftPixelX )
-        return KHE::KSection();
+        return KHE::Section();
 
     // translate
     PX -= x();
     const int PRX = PX + PW - 1;
 
-    KHE::KSection positions;
+    KHE::Section positions;
     // search backwards for the first byte that is equalleft to x
     for( int p=mLastLinePos; p>=0; --p )
         if( mLinePosLeftPixelX[p] <= PRX )
@@ -301,14 +301,14 @@ int AbstractByteArrayColumnRenderer::linePositionOfColumnX( KPixelX PX ) const
 }
 
 
-KHE::KSection AbstractByteArrayColumnRenderer::linePositionsOfColumnXs( KPixelX PX, KPixelX PW ) const
+KHE::Section AbstractByteArrayColumnRenderer::linePositionsOfColumnXs( KPixelX PX, KPixelX PW ) const
 {
     if( !mLinePosLeftPixelX )
-        return KHE::KSection();
+        return KHE::Section();
 
     const int PRX = PX + PW - 1;
 
-    KHE::KSection positions;
+    KHE::Section positions;
     // search backwards for the first byte that is equalleft to x
     for( int p=mLastLinePos; p>=0; --p )
         if( mLinePosLeftPixelX[p] <= PRX )
@@ -333,7 +333,7 @@ KPixelX AbstractByteArrayColumnRenderer::columnRightXOfLinePosition( int linePos
 { return mLinePosRightPixelX ? mLinePosRightPixelX[linePosition] : 0; }
 
 
-KPixelXs AbstractByteArrayColumnRenderer::xsOfLinePositionsInclSpaces( const KHE::KSection &linePositions ) const
+KPixelXs AbstractByteArrayColumnRenderer::xsOfLinePositionsInclSpaces( const KHE::Section &linePositions ) const
 {
     const int x = (linePositions.start()>0) ? rightXOfLinePosition( linePositions.nextBeforeStart() ) + 1 :
                                               xOfLinePosition( linePositions.start() );
@@ -343,7 +343,7 @@ KPixelXs AbstractByteArrayColumnRenderer::xsOfLinePositionsInclSpaces( const KHE
 }
 
 
-KPixelXs AbstractByteArrayColumnRenderer::columnXsOfLinePositionsInclSpaces( const KHE::KSection &linePositions ) const
+KPixelXs AbstractByteArrayColumnRenderer::columnXsOfLinePositionsInclSpaces( const KHE::Section &linePositions ) const
 {
     const int x = (linePositions.start()>0) ? columnRightXOfLinePosition( linePositions.nextBeforeStart() ) + 1 :
                                               columnXOfLinePosition( linePositions.start() );
@@ -385,7 +385,7 @@ void AbstractByteArrayColumnRenderer::renderNextLine( QPainter *painter )
 }
 
 
-void AbstractByteArrayColumnRenderer::renderLinePositions( QPainter *painter, int lineIndex, const KHE::KSection &_linePositions )
+void AbstractByteArrayColumnRenderer::renderLinePositions( QPainter *painter, int lineIndex, const KHE::Section &_linePositions )
 {
     // clear background
     const unsigned int blankFlag =
@@ -397,7 +397,7 @@ void AbstractByteArrayColumnRenderer::renderLinePositions( QPainter *painter, in
 
     // Go through the lines TODO: handle first and last line more effeciently
     // check for leading and trailing spaces
-    KHE::KSection linePositions( mLayout->firstLinePosition(Coord( _linePositions.start(), lineIndex )),
+    KHE::Section linePositions( mLayout->firstLinePosition(Coord( _linePositions.start(), lineIndex )),
                                  mLayout->lastLinePosition( Coord( _linePositions.end(),   lineIndex )) );
 
     // no bytes to paint?
@@ -405,13 +405,13 @@ void AbstractByteArrayColumnRenderer::renderLinePositions( QPainter *painter, in
         return;
 
     // check for leading and trailing spaces
-    KHE::KSection byteIndizes =
-        KHE::KSection::fromWidth( mLayout->indexAtCoord(Coord( linePositions.start(), lineIndex )), linePositions.width() );
+    KHE::Section byteIndizes =
+        KHE::Section::fromWidth( mLayout->indexAtCoord(Coord( linePositions.start(), lineIndex )), linePositions.width() );
 
     unsigned int selectionFlag = 0;
     unsigned int markingFlag = 0;
-    KHE::KSection selection;
-    KHE::KSection markedSection;
+    KHE::Section selection;
+    KHE::Section markedSection;
     bool hasMarking = mRanges->hasMarking();
     bool hasSelection = mRanges->hasSelection();
 
@@ -420,8 +420,8 @@ void AbstractByteArrayColumnRenderer::renderLinePositions( QPainter *painter, in
 //         <<" for byteIndizes "<<byteIndizes.start()<<"-"<<byteIndizes.start()<<endl;
     while( linePositions.isValid() )
     {
-        KHE::KSection positionsPart( linePositions );  // set of linePositions to paint next
-        KHE::KSection byteIndizesPart( byteIndizes );      // set of indizes to paint next
+        KHE::Section positionsPart( linePositions );  // set of linePositions to paint next
+        KHE::Section byteIndizesPart( byteIndizes );      // set of indizes to paint next
         // falls markedSection nicht mehr gebuffert und noch zu erwarten
         if( hasMarking && markedSection.endsBefore(byteIndizesPart.start()) )
         {
@@ -476,11 +476,11 @@ void AbstractByteArrayColumnRenderer::renderLinePositions( QPainter *painter, in
 }
 
 
-void AbstractByteArrayColumnRenderer::renderPlain( QPainter *painter, const KHE::KSection &linePositions, int byteIndex )
+void AbstractByteArrayColumnRenderer::renderPlain( QPainter *painter, const KHE::Section &linePositions, int byteIndex )
 {
     bool hasBookmarks = ( mBookmarks != 0 );
-    KHECore::KBookmarkList bookmarkList;
-    KHECore::KBookmarkList::ConstIterator bit;
+    KHECore::BookmarkList bookmarkList;
+    KHECore::BookmarkList::ConstIterator bit;
     if( hasBookmarks )
     {
         bookmarkList = mBookmarks->bookmarkList();
@@ -520,11 +520,11 @@ void AbstractByteArrayColumnRenderer::renderPlain( QPainter *painter, const KHE:
 }
 
 
-void AbstractByteArrayColumnRenderer::renderSelection( QPainter *painter, const KHE::KSection &linePositions, int byteIndex, int flag )
+void AbstractByteArrayColumnRenderer::renderSelection( QPainter *painter, const KHE::Section &linePositions, int byteIndex, int flag )
 {
     bool hasBookmarks = ( mBookmarks != 0 );
-    KHECore::KBookmarkList bookmarkList;
-    KHECore::KBookmarkList::ConstIterator bit;
+    KHECore::BookmarkList bookmarkList;
+    KHECore::BookmarkList::ConstIterator bit;
     if( hasBookmarks )
     {
         bookmarkList = mBookmarks->bookmarkList();
@@ -566,7 +566,7 @@ void AbstractByteArrayColumnRenderer::renderSelection( QPainter *painter, const
 }
 
 
-void AbstractByteArrayColumnRenderer::renderMarking( QPainter *painter, const KHE::KSection &linePositions, int byteIndex, int flag )
+void AbstractByteArrayColumnRenderer::renderMarking( QPainter *painter, const KHE::Section &linePositions, int byteIndex, int flag )
 {
     const QPalette &palette = columnsView()->viewport()->palette();
 
@@ -596,7 +596,7 @@ void AbstractByteArrayColumnRenderer::renderBookmark( QPainter *painter, const Q
 }
 
 
-void AbstractByteArrayColumnRenderer::renderRange( QPainter *painter, const QBrush &brush, const KHE::KSection &linePositions, int flag )
+void AbstractByteArrayColumnRenderer::renderRange( QPainter *painter, const QBrush &brush, const KHE::Section &linePositions, int flag )
 {
     const KPixelX rangeX =
         ( flag & StartsBefore ) ? columnRightXOfLinePosition( linePositions.nextBeforeStart() ) + 1 :
@@ -656,8 +656,11 @@ void AbstractByteArrayColumnRenderer::renderFramedByte( QPainter *painter, int b
     const char byte = ( byteIndex > -1 ) ? mByteArrayModel->datum( byteIndex ) : EmptyByte;
     const KHECore::KChar byteChar = mCharCodec->decode( byte );
 
+    const bool isInSelection = ( byteIndex > -1 && mRanges->selectionIncludes(byteIndex) );
+    const KColorScheme::ColorSet colorSet = isInSelection ? KColorScheme::Selection : KColorScheme::View;
+
     const QPalette &palette = columnsView()->viewport()->palette();
-    KColorScheme colorScheme( palette.currentColorGroup(), KColorScheme::View );
+    KColorScheme colorScheme( palette.currentColorGroup(), colorSet );
     const KColorScheme::ForegroundRole foregroundRole =
         mByteTypeColored ? foregroundRoleForChar(byteChar): KColorScheme::NormalText;
     const QBrush brush = colorScheme.foreground( foregroundRole );
@@ -677,8 +680,11 @@ void AbstractByteArrayColumnRenderer::renderCursor( QPainter *painter, int byteI
     const char byte = ( byteIndex > -1 ) ? mByteArrayModel->datum( byteIndex ) : EmptyByte;
     const KHECore::KChar byteChar = mCharCodec->decode( byte );
 
+    const bool isInSelection = ( byteIndex > -1 && mRanges->selectionIncludes(byteIndex) );
+    const KColorScheme::ColorSet colorSet = isInSelection ? KColorScheme::Selection : KColorScheme::View;
+
     const QPalette &palette = columnsView()->viewport()->palette();
-    KColorScheme colorScheme( palette.currentColorGroup(), KColorScheme::View );
+    KColorScheme colorScheme( palette.currentColorGroup(), colorSet );
     const KColorScheme::ForegroundRole foregroundRole =
         mByteTypeColored ? foregroundRoleForChar(byteChar): KColorScheme::NormalText;
     const QBrush brush = colorScheme.foreground( foregroundRole );
@@ -686,14 +692,14 @@ void AbstractByteArrayColumnRenderer::renderCursor( QPainter *painter, int byteI
 }
 
 
-bool AbstractByteArrayColumnRenderer::isSelected( const KHE::KSection &section, KHE::KSection *_selection,
+bool AbstractByteArrayColumnRenderer::isSelected( const KHE::Section &section, KHE::Section *_selection,
                                                   unsigned int *_flag ) const
 {
-    const KHE::KSection *overlappingSelectedSection = mRanges->firstOverlappingSelection( section );
+    const KHE::Section *overlappingSelectedSection = mRanges->firstOverlappingSelection( section );
     if( !overlappingSelectedSection )
         return false;
 
-    KHE::KSection selection = *overlappingSelectedSection;
+    KHE::Section selection = *overlappingSelectedSection;
     unsigned int flag = 0;
 
     // does selection start before asked range?
@@ -716,15 +722,15 @@ bool AbstractByteArrayColumnRenderer::isSelected( const KHE::KSection &section,
 }
 
 
-bool AbstractByteArrayColumnRenderer::isMarked( const KHE::KSection &section, KHE::KSection *_markedSection,
+bool AbstractByteArrayColumnRenderer::isMarked( const KHE::Section &section, KHE::Section *_markedSection,
                                                 unsigned int *_flag ) const
 {
-    const KHE::KSection *overlappingMarkedSection = mRanges->overlappingMarking( section );
+    const KHE::Section *overlappingMarkedSection = mRanges->overlappingMarking( section );
     if( !overlappingMarkedSection )
         return false;
 
     unsigned int flag = 0;
-    KHE::KSection markedSection = *overlappingMarkedSection;
+    KHE::Section markedSection = *overlappingMarkedSection;
 
     if( markedSection.startsBefore(section) )
     {
